@@ -33,6 +33,7 @@ from tensorflow.python.ops import linalg_ops
 from tensorflow.python.ops import math_ops
 from tensorflow.python.ops import random_ops
 from tensorflow.python.ops import state_ops
+from tensorflow.python.ops import variable_scope
 from tensorflow.python.ops import variables
 from tensorflow.python.ops.embedding_ops import embedding_lookup
 from tensorflow.python.summary import summary
@@ -85,13 +86,14 @@ def _init_clusters_random(data, num_clusters, random_seed):
         maxval=math_ops.cast(num_data, dtypes.int64),
         seed=random_seed,
         dtype=dtypes.int64)
-  indices = math_ops.cast(indices, dtypes.int32) % num_data
+  indices %= math_ops.cast(num_data, dtypes.int64)
   clusters_init = embedding_lookup(data, indices, partition_strategy='div')
   return clusters_init
 
 
 class GmmAlgorithm(object):
   """Tensorflow Gaussian mixture model clustering class."""
+  CLUSTERS_WEIGHT = 'alphas'
   CLUSTERS_VARIABLE = 'clusters'
   CLUSTERS_COVS_VARIABLE = 'clusters_covs'
 
@@ -143,15 +145,34 @@ class GmmAlgorithm(object):
     # Small value to guarantee that covariances are invertible.
     self._min_var = array_ops.diag(
         array_ops.ones(array_ops.stack([self._dimensions]))) * 1e-3
-    self._create_variables(data, initial_means)
+    self._create_variables()
+    self._initialize_variables(data, initial_means)
     # Operations of partial statistics for the computation of the means.
     self._w_mul_x = []
     # Operations of partial statistics for the computation of the covariances.
     self._w_mul_x2 = []
     self._define_graph(data)
 
-  def _create_variables(self, data, initial_means=None):
-    """Initializes GMM algorithm.
+  def _create_variables(self):
+    """Initializes GMM algorithm."""
+    init_value = array_ops.constant([], dtype=dtypes.float32)
+    self._means = variables.Variable(init_value,
+                                     name=self.CLUSTERS_VARIABLE,
+                                     validate_shape=False)
+    self._covs = variables.Variable(
+        init_value, name=self.CLUSTERS_COVS_VARIABLE, validate_shape=False)
+    # Mixture weights, representing the probability that a randomly
+    # selected unobservable data (in EM terms) was generated by component k.
+    self._alpha = variable_scope.variable(
+        array_ops.tile([1.0 / self._num_classes], [self._num_classes]),
+        name=self.CLUSTERS_WEIGHT,
+        validate_shape=False)
+    self._cluster_centers_initialized = variables.Variable(False,
+                                                           dtype=dtypes.bool,
+                                                           name='initialized')
+
+  def _initialize_variables(self, data, initial_means=None):
+    """Initializes variables.
 
     Args:
       data: a list of Tensors with data, each row is a new example.
@@ -160,19 +181,11 @@ class GmmAlgorithm(object):
     first_shard = data[0]
     # Initialize means: num_classes X 1 X dimensions.
     if initial_means is not None:
-      self._means = variables.Variable(
-          array_ops.expand_dims(initial_means, 1),
-          name=self.CLUSTERS_VARIABLE,
-          validate_shape=False,
-          dtype=dtypes.float32)
+      means = array_ops.expand_dims(initial_means, 1)
     else:
       # Sample data randomly
-      self._means = variables.Variable(
-          array_ops.expand_dims(
-              _init_clusters_random(data, self._num_classes, self._random_seed),
-              1),
-          name=self.CLUSTERS_VARIABLE,
-          validate_shape=False)
+      means = array_ops.expand_dims(
+          _init_clusters_random(data, self._num_classes, self._random_seed), 1)
 
     # Initialize covariances.
     if self._covariance_type == FULL_COVARIANCE:
@@ -186,16 +199,40 @@ class GmmAlgorithm(object):
       covs = array_ops.tile(
           array_ops.expand_dims(array_ops.diag_part(cov), 0),
           [self._num_classes, 1])
-    self._covs = variables.Variable(
-        covs, name='clusters_covs', validate_shape=False)
-    # Mixture weights, representing the probability that a randomly
-    # selected unobservable data (in EM terms) was generated by component k.
-    self._alpha = variables.Variable(
-        array_ops.tile([1.0 / self._num_classes], [self._num_classes]))
+
+    with ops.colocate_with(self._cluster_centers_initialized):
+      initialized = control_flow_ops.with_dependencies(
+          [means, covs],
+          array_ops.identity(self._cluster_centers_initialized))
+    self._init_ops = []
+    with ops.colocate_with(self._means):
+      init_means = state_ops.assign(self._means, means, validate_shape=False)
+      init_means = control_flow_ops.with_dependencies(
+          [init_means],
+          state_ops.assign(self._cluster_centers_initialized, True))
+      self._init_ops.append(control_flow_ops.cond(initialized,
+                                                  control_flow_ops.no_op,
+                                                  lambda: init_means).op)
+    with ops.colocate_with(self._covs):
+      init_covs = state_ops.assign(self._covs, covs, validate_shape=False)
+      init_covs = control_flow_ops.with_dependencies(
+          [init_covs],
+          state_ops.assign(self._cluster_centers_initialized, True))
+      self._init_ops.append(control_flow_ops.cond(initialized,
+                                                  control_flow_ops.no_op,
+                                                  lambda: init_covs).op)
+
+  def init_ops(self):
+    """Returns the initialization operation."""
+    return control_flow_ops.group(*self._init_ops)
 
   def training_ops(self):
     """Returns the training operation."""
-    return self._train_ops
+    return control_flow_ops.group(*self._train_ops)
+
+  def is_initialized(self):
+    """Returns a boolean operation for initialized variables."""
+    return self._cluster_centers_initialized
 
   def alphas(self):
     return self._alpha
@@ -312,9 +349,8 @@ class GmmAlgorithm(object):
     Args:
       shard_id: id of current shard_id.
     """
-    self._prior_probs[shard_id] = math_ops.log(
-        math_ops.reduce_sum(
-            math_ops.exp(self._probs[shard_id]), 1, keep_dims=True))
+    self._prior_probs[shard_id] = math_ops.reduce_logsumexp(
+        self._probs[shard_id], axis=1, keep_dims=True)
 
   def _define_expectation_operation(self, shard_id):
     # Shape broadcasting.
@@ -484,6 +520,7 @@ def gmm(inp,
     scores: Similar to assignments but specifies the distance to the
       assigned cluster instead.
     training_op: an op that runs an iteration of training.
+    init_op: an op that runs the initialization.
   """
   initial_means = None
   if initial_clusters != 'random' and not isinstance(initial_clusters,
@@ -494,8 +531,7 @@ def gmm(inp,
   inp = inp if isinstance(inp, list) else [inp]
   gmm_tool = GmmAlgorithm(inp, num_clusters, initial_means, params,
                           covariance_type, random_seed)
-  training_ops = gmm_tool.training_ops()
   assignments = gmm_tool.assignments()
   all_scores, scores = gmm_tool.scores()
-  return [all_scores], [assignments], [scores], control_flow_ops.group(
-      *training_ops)
+  return ([all_scores], [assignments], [scores], gmm_tool.training_ops(),
+          gmm_tool.init_ops(), gmm_tool.is_initialized())
