@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-
 """Helpers to manipulate a tensor graph in python.
 """
 
@@ -21,6 +20,7 @@ from __future__ import division
 from __future__ import print_function
 import copy
 import re
+import six
 
 from tensorflow.core.framework import attr_value_pb2
 from tensorflow.core.framework import graph_pb2
@@ -79,7 +79,7 @@ def must_run_on_cpu(node, pin_variables_on_cpu=False):
     if dtype == dtypes.string or dtype == dtypes.int32:
       return True
 
-  if node_def.op == "DynamicStitch":
+  if node_def.op in ["DynamicStitch", "ParallelDynamicStitch"]:
     dtype = node_def.attr["T"].type
     if dtype == dtypes.int32:
       # DynamicStitch on GPU only works for int32 values.
@@ -107,6 +107,46 @@ def _node_name(n):
     return n.split(":")[0]
 
 
+def _extract_graph_summary(graph_def):
+  """Extracts useful information from the graph and returns them."""
+  name_to_input_name = {}  # Keyed by the dest node name.
+  name_to_node = {}  # Keyed by node name.
+
+  # Keeps track of node sequences. It is important to still output the
+  # operations in the original order.
+  name_to_seq_num = {}  # Keyed by node name.
+  seq = 0
+  for node in graph_def.node:
+    n = _node_name(node.name)
+    name_to_node[n] = node
+    name_to_input_name[n] = [_node_name(x) for x in node.input]
+    name_to_seq_num[n] = seq
+    seq += 1
+  return name_to_input_name, name_to_node, name_to_seq_num
+
+
+def _assert_nodes_are_present(name_to_node, nodes):
+  """Assert that nodes are present in the graph."""
+  for d in nodes:
+    assert d in name_to_node, "%s is not in graph" % d
+
+
+def _bfs_for_reachable_nodes(target_nodes, name_to_input_name):
+  """Breadth first search for reachable nodes from target nodes."""
+  nodes_to_keep = set()
+  # Breadth first search to find all the nodes that we should keep.
+  next_to_visit = target_nodes[:]
+  while next_to_visit:
+    n = next_to_visit[0]
+    del next_to_visit[0]
+    if n in nodes_to_keep:
+      # Already visited this node.
+      continue
+    nodes_to_keep.add(n)
+    next_to_visit += name_to_input_name[n]
+  return nodes_to_keep
+
+
 def extract_sub_graph(graph_def, dest_nodes):
   """Extract the subgraph that can reach any of the nodes in 'dest_nodes'.
 
@@ -123,40 +163,23 @@ def extract_sub_graph(graph_def, dest_nodes):
   if not isinstance(graph_def, graph_pb2.GraphDef):
     raise TypeError("graph_def must be a graph_pb2.GraphDef proto.")
 
-  edges = {}  # Keyed by the dest node name.
-  name_to_node_map = {}  # Keyed by node name.
+  if isinstance(dest_nodes, six.string_types):
+    raise TypeError("dest_nodes must be a list.")
 
-  # Keeps track of node sequences. It is important to still output the
-  # operations in the original order.
-  node_seq = {}  # Keyed by node name.
-  seq = 0
-  for node in graph_def.node:
-    n = _node_name(node.name)
-    name_to_node_map[n] = node
-    edges[n] = [_node_name(x) for x in node.input]
-    node_seq[n] = seq
-    seq += 1
+  name_to_input_name, name_to_node, name_to_seq_num = _extract_graph_summary(
+      graph_def)
+  _assert_nodes_are_present(name_to_node, dest_nodes)
 
-  for d in dest_nodes:
-    assert d in name_to_node_map, "%s is not in graph" % d
+  nodes_to_keep = _bfs_for_reachable_nodes(dest_nodes, name_to_input_name)
 
-  nodes_to_keep = set()
-  # Breadth first search to find all the nodes that we should keep.
-  next_to_visit = dest_nodes[:]
-  while next_to_visit:
-    n = next_to_visit[0]
-    del next_to_visit[0]
-    if n in nodes_to_keep:
-      # Already visited this node.
-      continue
-    nodes_to_keep.add(n)
-    next_to_visit += edges[n]
-
-  nodes_to_keep_list = sorted(list(nodes_to_keep), key=lambda n: node_seq[n])
+  nodes_to_keep_list = sorted(
+      list(nodes_to_keep), key=lambda n: name_to_seq_num[n])
   # Now construct the output GraphDef
   out = graph_pb2.GraphDef()
   for n in nodes_to_keep_list:
-    out.node.extend([copy.deepcopy(name_to_node_map[n])])
+    out.node.extend([copy.deepcopy(name_to_node[n])])
+  out.library.CopyFrom(graph_def.library)
+  out.versions.CopyFrom(graph_def.versions)
 
   return out
 
@@ -175,7 +198,9 @@ def tensor_shape_from_node_def_name(graph, input_name):
   return shape
 
 
-def convert_variables_to_constants(sess, input_graph_def, output_node_names,
+def convert_variables_to_constants(sess,
+                                   input_graph_def,
+                                   output_node_names,
                                    variable_names_whitelist=None,
                                    variable_names_blacklist=None):
   """Replaces all the variables in a graph with constants of the same values.
@@ -219,7 +244,7 @@ def convert_variables_to_constants(sess, input_graph_def, output_node_names,
   else:
     returned_variables = []
   found_variables = dict(zip(variable_dict_names, returned_variables))
-  logging.info("Froze %d variables." % len(returned_variables))
+  logging.info("Froze %d variables.", len(returned_variables))
 
   output_graph_def = graph_pb2.GraphDef()
   how_many_converted = 0
@@ -231,19 +256,21 @@ def convert_variables_to_constants(sess, input_graph_def, output_node_names,
       dtype = input_node.attr["dtype"]
       data = found_variables[input_node.name]
       output_node.attr["dtype"].CopyFrom(dtype)
-      output_node.attr["value"].CopyFrom(attr_value_pb2.AttrValue(
-          tensor=tensor_util.make_tensor_proto(data,
-                                               dtype=dtype.type,
-                                               shape=data.shape)))
+      output_node.attr["value"].CopyFrom(
+          attr_value_pb2.AttrValue(
+              tensor=tensor_util.make_tensor_proto(
+                  data, dtype=dtype.type, shape=data.shape)))
       how_many_converted += 1
     else:
       output_node.CopyFrom(input_node)
     output_graph_def.node.extend([output_node])
+
+  output_graph_def.library.CopyFrom(inference_graph.library)
   print("Converted %d variables to const ops." % how_many_converted)
   return output_graph_def
 
 
-def remove_training_nodes(input_graph):
+def remove_training_nodes(input_graph, protected_nodes=None):
   """Prunes out nodes that aren't needed for inference.
 
   There are nodes like Identity and CheckNumerics that are only useful
@@ -255,17 +282,22 @@ def remove_training_nodes(input_graph):
 
   Args:
     input_graph: Model to analyze and prune.
+    protected_nodes: An optional list of names of nodes to be kept
+      unconditionally. This is for example useful to preserve Identity output
+      nodes.
 
   Returns:
     A list of nodes with the unnecessary ones removed.
   """
+  if not protected_nodes:
+    protected_nodes = []
 
   types_to_remove = {"CheckNumerics": True}
 
   input_nodes = input_graph.node
   names_to_remove = {}
   for node in input_nodes:
-    if node.op in types_to_remove:
+    if node.op in types_to_remove and node.name not in protected_nodes:
       names_to_remove[node.name] = True
 
   nodes_after_removal = []
@@ -286,7 +318,7 @@ def remove_training_nodes(input_graph):
   types_to_splice = {"Identity": True}
   names_to_splice = {}
   for node in nodes_after_removal:
-    if node.op in types_to_splice:
+    if node.op in types_to_splice and node.name not in protected_nodes:
       # We don't want to remove nodes that have control edge inputs, because
       # they might be involved in subtle dependency issues that removing them
       # will jeopardize.
