@@ -26,6 +26,7 @@ import numpy as np
 
 from tensorflow.python.eager import context
 from tensorflow.python.framework import constant_op
+from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import ops
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.layers import base
@@ -49,9 +50,14 @@ class BatchNormalization(base.Layer):
   Sergey Ioffe, Christian Szegedy
 
   Arguments:
-    axis: Integer, the axis that should be normalized (typically the features
-      axis). For instance, after a `Conv2D` layer with
-      `data_format="channels_first"`, set `axis=1` in `BatchNormalization`.
+    axis: An `int` or list of `int`, the axis or axes that should be
+        normalized, typically the features axis/axes. For instance, after a
+        `Conv2D` layer with `data_format="channels_first"`, set `axis=1`. If a
+        list of axes is provided, each axis in `axis` will be normalized
+        simultaneously. Default is `-1` which takes uses last axis. Note: when
+        using multi-axis batch norm, the `beta`, `gamma`, `moving_mean`, and
+        `moving_variance` variables are the same rank as the input Tensor, with
+        dimension size 1 in all reduced (non-axis) dimensions).
     momentum: Momentum for the moving average.
     epsilon: Small float added to variance to avoid dividing by zero.
     center: If True, add offset of `beta` to normalized tensor. If False, `beta`
@@ -90,6 +96,24 @@ class BatchNormalization(base.Layer):
       If `None`, use the system recommended implementation.
     trainable: Boolean, if `True` also add variables to the graph collection
       `GraphKeys.TRAINABLE_VARIABLES` (see tf.Variable).
+    virtual_batch_size: An `int`. By default, `virtual_batch_size` is `None`,
+      which means batch normalization is performed across the whole batch. When
+      `virtual_batch_size` is not `None`, instead perform "Ghost Batch
+      Normalization", which creates virtual sub-batches which are each
+      normalized separately (with shared gamma, beta, and moving statistics).
+      Must divide the actual batch size during execution.
+    adjustment: A function taking the `Tensor` containing the (dynamic) shape of
+      the input tensor and returning a pair (scale, bias) to apply to the
+      normalized values (before gamma and beta), only during training. For
+      example, if axis==-1,
+        `adjustment = lambda shape: (
+          tf.random_uniform(shape[-1:], 0.93, 1.07),
+          tf.random_uniform(shape[-1:], -0.1, 0.1))`
+      will scale the normalized value by up to 7% up or down, then shift the
+      result by up to 0.1 (with independent scaling and bias for each feature
+      but shared across all examples), and finally apply gamma and/or beta. If
+      `None`, no adjustment is applied. Cannot be specified if
+      virtual_batch_size is specified.
     name: A string, the name of the layer.
   """
 
@@ -112,11 +136,16 @@ class BatchNormalization(base.Layer):
                renorm_momentum=0.99,
                fused=None,
                trainable=True,
+               virtual_batch_size=None,
+               adjustment=None,
                name=None,
                **kwargs):
     super(BatchNormalization, self).__init__(
         name=name, trainable=trainable, **kwargs)
-    self.axis = axis
+    if isinstance(axis, list):
+      self.axis = axis[:]
+    else:
+      self.axis = axis
     self.momentum = momentum
     self.epsilon = epsilon
     self.center = center
@@ -130,11 +159,14 @@ class BatchNormalization(base.Layer):
     self.beta_constraint = beta_constraint
     self.gamma_constraint = gamma_constraint
     self.renorm = renorm
+    self.virtual_batch_size = virtual_batch_size
+    self.adjustment = adjustment
     if fused is None:
       fused = True
 
     self.fused = fused
     self._bessels_correction_test_only = True
+
     if renorm:
       renorm_clipping = renorm_clipping or {}
       keys = ['rmax', 'rmin', 'dmax']
@@ -148,61 +180,121 @@ class BatchNormalization(base.Layer):
     input_shape = tensor_shape.TensorShape(input_shape)
     if not input_shape.ndims:
       raise ValueError('Input has undefined rank:', input_shape)
-    ndim = len(input_shape)
-    if self.axis < 0:
-      axis = ndim + self.axis
-    else:
-      axis = self.axis
-    if axis < 0 or axis >= ndim:
-      raise ValueError('Value of `axis` argument ' + str(self.axis) +
-                       ' is out of range for input with rank ' + str(ndim))
+    ndims = len(input_shape)
+
+    # Convert axis to list and resolve negatives
+    if isinstance(self.axis, int):
+      self.axis = [self.axis]
+
+    if not isinstance(self.axis, list):
+      raise TypeError('axis must be int or list, type given: %s'
+                      % type(self.axis))
+
+    for idx, x in enumerate(self.axis):
+      if x < 0:
+        self.axis[idx] = ndims + x
+
+    # Validate axes
+    for x in self.axis:
+      if x < 0 or x >= ndims:
+        raise ValueError('Invalid axis: %d' % x)
+    if len(self.axis) != len(set(self.axis)):
+      raise ValueError('Duplicate axis: %s' % self.axis)
+
+    if self.virtual_batch_size is not None:
+      if self.virtual_batch_size <= 0:
+        raise ValueError('virtual_batch_size must be a positive integer that '
+                         'divides the true batch size of the input Tensor')
+      # If using virtual batches, the first dimension must be the batch
+      # dimension and cannot be the batch norm axis
+      if 0 in self.axis:
+        raise ValueError('When using virtual_batch_size, the batch dimension '
+                         'must be 0 and thus axis cannot include 0')
+      if self.adjustment is not None:
+        raise ValueError('When using virtual_batch_size, adjustment cannot '
+                         'be specified')
 
     if self.fused:
-      # Currently fused batch norm doesn't support renorm and beta/gamma
-      # regularizer; and only supports an input tensor of rank 4 and a channel
-      # dimension on axis 1 and 3.
+      # Currently fused batch norm doesn't support renorm. It also only supports
+      # an input tensor of rank 4 and a channel dimension on axis 1 or 3.
       # TODO(yaozhang): if input is not 4D, reshape it to 4D and reshape the
       # output back to its original shape accordingly.
-      self.fused = not self.renorm and ndim == 4 and axis in [
-          1, 3
-      ] and self.beta_regularizer is None and self.gamma_regularizer is None
+      self.fused = (not self.renorm and
+                    ndims == 4 and
+                    self.axis in [[1], [3]] and
+                    self.virtual_batch_size is None and
+                    self.adjustment is None)
+      # TODO(chrisying): fused batch norm is currently not supported for
+      # multi-axis batch norm and by extension virtual batches. In some cases,
+      # it might be possible to use fused batch norm but would require reshaping
+      # the Tensor to 4D with the axis in 1 or 3 (preferred 1) which is
+      # particularly tricky. A compromise might be to just support the most
+      # common use case (turning 5D w/ virtual batch to NCHW)
 
     if self.fused:
-      if axis == 1:
+      if self.axis == [1]:
         self._data_format = 'NCHW'
-      else:
+      elif self.axis == [3]:
         self._data_format = 'NHWC'
+      else:
+        raise ValueError('Unsupported axis, fused batch norm only supports '
+                         'axis == [1] or axis == [3]')
 
-    param_dim = input_shape[axis]
-    if not param_dim.value:
-      raise ValueError('Input has undefined `axis` dimension. Input shape: ',
-                       input_shape)
-    self.input_spec = base.InputSpec(ndim=ndim,
-                                     axes={self.axis: param_dim.value})
+    # Raise parameters of fp16 batch norm to fp32
+    if self.dtype == dtypes.float16 or self.dtype == dtypes.bfloat16:
+      param_dtype = dtypes.float32
+    else:
+      param_dtype = self.dtype or dtypes.float32
+
+    axis_to_dim = {x: input_shape[x].value for x in self.axis}
+    for x in axis_to_dim:
+      if axis_to_dim[x] is None:
+        raise ValueError('Input has undefined `axis` dimension. Input shape: ',
+                         input_shape)
+    self.input_spec = base.InputSpec(ndim=ndims, axes=axis_to_dim)
+
+    if len(axis_to_dim) == 1 and self.virtual_batch_size is None:
+      # Single axis batch norm (most common/default use-case)
+      param_shape = (list(axis_to_dim.values())[0],)
+    else:
+      # Parameter shape is the original shape but with 1 in all non-axis dims
+      param_shape = [axis_to_dim[i] if i in axis_to_dim
+                     else 1 for i in range(ndims)]
+      if self.virtual_batch_size is not None:
+        # When using virtual batches, add an extra dim at index 1
+        param_shape.insert(1, 1)
+        for idx, x in enumerate(self.axis):
+          self.axis[idx] = x + 1      # Account for added dimension
 
     if self.scale:
-      self.gamma = self.add_variable(name='gamma',
-                                     shape=(param_dim,),
-                                     initializer=self.gamma_initializer,
-                                     regularizer=self.gamma_regularizer,
-                                     constraint=self.gamma_constraint,
-                                     trainable=True)
+      self.gamma = self.add_variable(
+          name='gamma',
+          shape=param_shape,
+          dtype=param_dtype,
+          initializer=self.gamma_initializer,
+          regularizer=self.gamma_regularizer,
+          constraint=self.gamma_constraint,
+          trainable=True)
     else:
       self.gamma = None
       if self.fused:
-        self._gamma_const = array_ops.constant(1.0, shape=(param_dim,))
+        self._gamma_const = array_ops.constant(
+            1.0, dtype=param_dtype, shape=param_shape)
 
     if self.center:
-      self.beta = self.add_variable(name='beta',
-                                    shape=(param_dim,),
-                                    initializer=self.beta_initializer,
-                                    regularizer=self.beta_regularizer,
-                                    constraint=self.beta_constraint,
-                                    trainable=True)
+      self.beta = self.add_variable(
+          name='beta',
+          shape=param_shape,
+          dtype=param_dtype,
+          initializer=self.beta_initializer,
+          regularizer=self.beta_regularizer,
+          constraint=self.beta_constraint,
+          trainable=True)
     else:
       self.beta = None
       if self.fused:
-        self._beta_const = array_ops.constant(0.0, shape=(param_dim,))
+        self._beta_const = array_ops.constant(
+            0.0, dtype=param_dtype, shape=param_shape)
 
     # Disable variable partitioning when creating the moving mean and variance
     try:
@@ -213,14 +305,18 @@ class BatchNormalization(base.Layer):
         partitioner = None
       self.moving_mean = self.add_variable(
           name='moving_mean',
-          shape=(param_dim,),
+          shape=param_shape,
+          dtype=param_dtype,
           initializer=self.moving_mean_initializer,
           trainable=False)
+
       self.moving_variance = self.add_variable(
           name='moving_variance',
-          shape=(param_dim,),
+          shape=param_shape,
+          dtype=param_dtype,
           initializer=self.moving_variance_initializer,
           trainable=False)
+
       self._one_minus_decay = 1.0 - self.momentum
       if self.renorm:
         # Create variables to maintain the moving mean and standard deviation.
@@ -231,17 +327,19 @@ class BatchNormalization(base.Layer):
         # stack to be cleared. The nested ones use a `lambda` to set the desired
         # device and ignore any devices that may be set by the custom getter.
         def _renorm_variable(name, shape):
-          var = self.add_variable(name=name,
-                                  shape=shape,
-                                  initializer=init_ops.zeros_initializer(),
-                                  trainable=False)
+          var = self.add_variable(
+              name=name,
+              shape=shape,
+              dtype=param_dtype,
+              initializer=init_ops.zeros_initializer(),
+              trainable=False)
           return var
 
         with ops.device(None):
           device = ((lambda _: self.moving_mean.device)
                     if context.in_graph_mode() else self.moving_mean.device)
           with ops.device(device):
-            self.renorm_mean = _renorm_variable('renorm_mean', (param_dim,))
+            self.renorm_mean = _renorm_variable('renorm_mean', param_shape)
             self.renorm_mean_weight = _renorm_variable('renorm_mean_weight', ())
           # We initialize renorm_stddev to 0, and maintain the (0-initialized)
           # renorm_stddev_weight. This allows us to (1) mix the average
@@ -250,7 +348,7 @@ class BatchNormalization(base.Layer):
           device = ((lambda _: self.moving_variance.device)
                     if context.in_graph_mode() else self.moving_variance.device)
           with ops.device(device):
-            self.renorm_stddev = _renorm_variable('renorm_stddev', (param_dim,))
+            self.renorm_stddev = _renorm_variable('renorm_stddev', param_shape)
             self.renorm_stddev_weight = _renorm_variable(
                 'renorm_stddev_weight', ())
     finally:
@@ -275,7 +373,6 @@ class BatchNormalization(base.Layer):
 
   def _fused_batch_norm(self, inputs, training):
     """Returns the output of fused batch norm."""
-    # TODO(reedwm): Add support for fp16 inputs.
     beta = self.beta if self.center else self._beta_const
     gamma = self.gamma if self.scale else self._gamma_const
 
@@ -311,9 +408,9 @@ class BatchNormalization(base.Layer):
 
     training_value = utils.constant_value(training)
     if training_value is None:
-      one_minus_decay = _smart_select(training,
-                                      lambda: self._one_minus_decay,
-                                      lambda: 0.)
+      one_minus_decay = utils.smart_cond(training,
+                                         lambda: self._one_minus_decay,
+                                         lambda: 0.)
     else:
       one_minus_decay = ops.convert_to_tensor(self._one_minus_decay)
     if training_value or training_value is None:
@@ -355,27 +452,30 @@ class BatchNormalization(base.Layer):
     if dmax is not None:
       d = math_ops.maximum(d, -dmax)
       d = math_ops.minimum(d, dmax)
-    # When not training, use r=1, d=0, and decay=1 meaning no updates.
-    r = _smart_select(training, lambda: r, lambda: array_ops.ones_like(r))
-    d = _smart_select(training, lambda: d, lambda: array_ops.zeros_like(d))
-    decay = _smart_select(training, lambda: self.renorm_momentum, lambda: 1.)
+    # When not training, use r=1, d=0.
+    r = utils.smart_cond(training, lambda: r, lambda: array_ops.ones_like(r))
+    d = utils.smart_cond(training, lambda: d, lambda: array_ops.zeros_like(d))
 
     def _update_renorm_variable(var, weight, value):
       """Updates a moving average and weight, returns the unbiased value."""
-      # Update the variables without zero debiasing. The debiasing will be
-      # accomplished by dividing the exponential moving average by the weight.
-      # For example, after a single update, the moving average would be
-      # (1-decay) * value. and the weight will be 1-decay, with their ratio
-      # giving value.
-      # Make sure the weight is not updated until before r and d computation.
       value = array_ops.identity(value)
-      with ops.control_dependencies([value]):
-        weight_value = array_ops.constant(1., dtype=weight.dtype)
-      new_var = moving_averages.assign_moving_average(
-          var, value, decay, zero_debias=False)
-      new_weight = moving_averages.assign_moving_average(
-          weight, weight_value, decay, zero_debias=False)
-      return new_var / new_weight
+      def _do_update():
+        # Update the variables without zero debiasing. The debiasing will be
+        # accomplished by dividing the exponential moving average by the weight.
+        # For example, after a single update, the moving average would be
+        # (1-decay) * value. and the weight will be 1-decay, with their ratio
+        # giving the value.
+        # Make sure the weight is not updated until before r and d computation.
+        with ops.control_dependencies([value]):
+          weight_value = array_ops.constant(1., dtype=weight.dtype)
+        new_var = moving_averages.assign_moving_average(
+            var, value, self.renorm_momentum, zero_debias=False)
+        new_weight = moving_averages.assign_moving_average(
+            weight, weight_value, self.renorm_momentum, zero_debias=False)
+        return new_var / new_weight
+      def _fake_update():
+        return array_ops.identity(var)
+      return utils.smart_cond(training, _do_update, _fake_update)
 
     with ops.colocate_with(self.moving_mean):
       new_mean = _update_renorm_variable(self.renorm_mean,
@@ -391,35 +491,83 @@ class BatchNormalization(base.Layer):
     return (r, d, new_mean, new_variance)
 
   def call(self, inputs, training=False):
+    if self.virtual_batch_size is not None:
+      # Virtual batches (aka ghost batches) can be simulated by reshaping the
+      # Tensor and reusing the existing batch norm implementation
+      original_shape = [-1] + inputs.shape.as_list()[1:]
+      expanded_shape = [self.virtual_batch_size, -1] + original_shape[1:]
+
+      # Will cause errors if virtual_batch_size does not divide the batch size
+      inputs = array_ops.reshape(inputs, expanded_shape)
+
+      def undo_virtual_batching(outputs):
+        outputs = array_ops.reshape(outputs, original_shape)
+        return outputs
+
     if self.fused:
-      return self._fused_batch_norm(inputs, training=training)
+      outputs = self._fused_batch_norm(inputs, training=training)
+      if self.virtual_batch_size is not None:
+        # Currently never reaches here since fused_batch_norm does not support
+        # virtual batching
+        return undo_virtual_batching(outputs)
+      return outputs
 
-    # First, compute the axes along which to reduce the mean / variance,
-    # as well as the broadcast shape to be used for all parameters.
+    # Compute the axes along which to reduce the mean / variance
     input_shape = inputs.get_shape()
-    ndim = len(input_shape)
-    reduction_axes = list(range(len(input_shape)))
-    del reduction_axes[self.axis]
-    broadcast_shape = [1] * len(input_shape)
-    broadcast_shape[self.axis] = input_shape[self.axis].value
+    ndims = len(input_shape)
+    reduction_axes = [i for i in range(ndims) if i not in self.axis]
+    if self.virtual_batch_size is not None:
+      del reduction_axes[1]     # Do not reduce along virtual batch dim
 
-    # Determines whether broadcasting is needed.
-    needs_broadcasting = (sorted(reduction_axes) != list(range(ndim))[:-1])
+    # Broadcasting only necessary for single-axis batch norm where the axis is
+    # not the last dimension
+    broadcast_shape = [1] * ndims
+    broadcast_shape[self.axis[0]] = input_shape[self.axis[0]].value
+    def _broadcast(v):
+      if (v is not None and
+          len(v.get_shape()) != ndims and
+          reduction_axes != list(range(ndims - 1))):
+        return array_ops.reshape(v, broadcast_shape)
+      return v
 
-    scale, offset = self.gamma, self.beta
+    scale, offset = _broadcast(self.gamma), _broadcast(self.beta)
+
+    def _compose_transforms(scale, offset, then_scale, then_offset):
+      if then_scale is not None:
+        scale *= then_scale
+        offset *= then_scale
+      if then_offset is not None:
+        offset += then_offset
+      return (scale, offset)
 
     # Determine a boolean value for `training`: could be True, False, or None.
     training_value = utils.constant_value(training)
     if training_value is not False:
+      if self.adjustment:
+        adj_scale, adj_bias = self.adjustment(array_ops.shape(inputs))
+        # Adjust only during training.
+        adj_scale = utils.smart_cond(training,
+                                     lambda: adj_scale,
+                                     lambda: array_ops.ones_like(adj_scale))
+        adj_bias = utils.smart_cond(training,
+                                    lambda: adj_bias,
+                                    lambda: array_ops.zeros_like(adj_bias))
+        scale, offset = _compose_transforms(adj_scale, adj_bias, scale, offset)
+
       # Some of the computations here are not necessary when training==False
       # but not a constant. However, this makes the code simpler.
-      mean, variance = nn.moments(inputs, reduction_axes)
-      mean = _smart_select(training,
-                           lambda: mean,
-                           lambda: self.moving_mean)
-      variance = _smart_select(training,
-                               lambda: variance,
-                               lambda: self.moving_variance)
+      keep_dims = self.virtual_batch_size is not None or len(self.axis) > 1
+      mean, variance = nn.moments(inputs, reduction_axes, keep_dims=keep_dims)
+
+      moving_mean = self.moving_mean
+      moving_variance = self.moving_variance
+
+      mean = utils.smart_cond(training,
+                              lambda: mean,
+                              lambda: moving_mean)
+      variance = utils.smart_cond(training,
+                                  lambda: variance,
+                                  lambda: moving_variance)
 
       if self.renorm:
         r, d, new_mean, new_variance = self._renorm_correction_and_moments(
@@ -427,22 +575,35 @@ class BatchNormalization(base.Layer):
         # When training, the normalized values (say, x) will be transformed as
         # x * gamma + beta without renorm, and (x * r + d) * gamma + beta
         # = x * (r * gamma) + (d * gamma + beta) with renorm.
-        scale = array_ops.stop_gradient(r, name='renorm_r')
-        offset = array_ops.stop_gradient(d, name='renorm_d')
-        if self.gamma is not None:
-          scale *= self.gamma
-          offset *= self.gamma
-        if self.beta is not None:
-          offset += self.beta
+        r = _broadcast(array_ops.stop_gradient(r, name='renorm_r'))
+        d = _broadcast(array_ops.stop_gradient(d, name='renorm_d'))
+        scale, offset = _compose_transforms(r, d, scale, offset)
       else:
         new_mean, new_variance = mean, variance
 
-      # Update moving averages when training, and prevent updates otherwise.
-      decay = _smart_select(training, lambda: self.momentum, lambda: 1.)
-      mean_update = moving_averages.assign_moving_average(
-          self.moving_mean, new_mean, decay, zero_debias=False)
-      variance_update = moving_averages.assign_moving_average(
-          self.moving_variance, new_variance, decay, zero_debias=False)
+      if self.virtual_batch_size is not None:
+        # This isn't strictly correct since in ghost batch norm, you are
+        # supposed to sequentially update the moving_mean and moving_variance
+        # with each sub-batch. However, since the moving statistics are only
+        # used during evaluation, it is more efficient to just update in one
+        # step and should not make a significant difference in the result.
+        new_mean = math_ops.reduce_mean(new_mean,
+                                        axis=1, keep_dims=True)
+        new_variance = math_ops.reduce_mean(new_variance,
+                                            axis=1, keep_dims=True)
+
+      def _do_update(var, value):
+        return moving_averages.assign_moving_average(
+            var, value, self.momentum, zero_debias=False)
+
+      mean_update = utils.smart_cond(
+          training,
+          lambda: _do_update(self.moving_mean, new_mean),
+          lambda: self.moving_mean)
+      variance_update = utils.smart_cond(
+          training,
+          lambda: _do_update(self.moving_variance, new_variance),
+          lambda: self.moving_variance)
       if context.in_graph_mode():
         self.add_update(mean_update, inputs=inputs)
         self.add_update(variance_update, inputs=inputs)
@@ -450,18 +611,19 @@ class BatchNormalization(base.Layer):
     else:
       mean, variance = self.moving_mean, self.moving_variance
 
-    def _broadcast(v):
-      if needs_broadcasting and v is not None:
-        # In this case we must explicitly broadcast all parameters.
-        return array_ops.reshape(v, broadcast_shape)
-      return v
+    outputs = nn.batch_normalization(inputs,
+                                     _broadcast(mean),
+                                     _broadcast(variance),
+                                     offset,
+                                     scale,
+                                     self.epsilon)
+    # If some components of the shape got lost due to adjustments, fix that.
+    outputs.set_shape(input_shape)
 
-    return nn.batch_normalization(inputs,
-                                  _broadcast(mean),
-                                  _broadcast(variance),
-                                  _broadcast(offset),
-                                  _broadcast(scale),
-                                  self.epsilon)
+    if self.virtual_batch_size is not None:
+      return undo_virtual_batching(outputs)
+
+    return outputs
 
 
 def batch_normalization(inputs,
@@ -485,7 +647,9 @@ def batch_normalization(inputs,
                         renorm=False,
                         renorm_clipping=None,
                         renorm_momentum=0.99,
-                        fused=None):
+                        fused=None,
+                        virtual_batch_size=None,
+                        adjustment=None):
   """Functional interface for the batch normalization layer.
 
   Reference: http://arxiv.org/abs/1502.03167
@@ -507,7 +671,7 @@ def batch_normalization(inputs,
 
   Arguments:
     inputs: Tensor input.
-    axis: Integer, the axis that should be normalized (typically the features
+    axis: An `int`, the axis that should be normalized (typically the features
       axis). For instance, after a `Convolution2D` layer with
       `data_format="channels_first"`, set `axis=1` in `BatchNormalization`.
     momentum: Momentum for the moving average.
@@ -557,9 +721,30 @@ def batch_normalization(inputs,
       to get the means and variances for inference.
     fused: if `True`, use a faster, fused implementation if possible.
       If `None`, use the system recommended implementation.
+    virtual_batch_size: An `int`. By default, `virtual_batch_size` is `None`,
+      which means batch normalization is performed across the whole batch. When
+      `virtual_batch_size` is not `None`, instead perform "Ghost Batch
+      Normalization", which creates virtual sub-batches which are each
+      normalized separately (with shared gamma, beta, and moving statistics).
+      Must divide the actual batch size during execution.
+    adjustment: A function taking the `Tensor` containing the (dynamic) shape of
+      the input tensor and returning a pair (scale, bias) to apply to the
+      normalized values (before gamma and beta), only during training. For
+      example, if axis==-1,
+        `adjustment = lambda shape: (
+          tf.random_uniform(shape[-1:], 0.93, 1.07),
+          tf.random_uniform(shape[-1:], -0.1, 0.1))`
+      will scale the normalized value by up to 7% up or down, then shift the
+      result by up to 0.1 (with independent scaling and bias for each feature
+      but shared across all examples), and finally apply gamma and/or beta. If
+      `None`, no adjustment is applied. Cannot be specified if
+      virtual_batch_size is specified.
 
   Returns:
     Output tensor.
+
+  Raises:
+    ValueError: if eager execution is enabled.
   """
   layer = BatchNormalization(
       axis=axis,
@@ -580,7 +765,10 @@ def batch_normalization(inputs,
       renorm_momentum=renorm_momentum,
       fused=fused,
       trainable=trainable,
+      virtual_batch_size=virtual_batch_size,
+      adjustment=adjustment,
       name=name,
+      dtype=inputs.dtype.base_dtype,
       _reuse=reuse,
       _scope=name)
   return layer.apply(inputs, training=training)
@@ -590,38 +778,3 @@ def batch_normalization(inputs,
 
 BatchNorm = BatchNormalization
 batch_norm = batch_normalization
-
-# Helper function
-
-
-def _smart_select(pred, fn_then, fn_else):
-  """Selects fn_then() or fn_else() based on the value of pred.
-
-  The purpose of this function is the same as `utils.smart_cond`. However, at
-  the moment there is a bug (b/36297356) that seems to kick in only when
-  `smart_cond` delegates to `tf.cond`, which sometimes results in the training
-  hanging when using parameter servers. This function will output the result
-  of `fn_then` or `fn_else` if `pred` is known at graph construction time.
-  Otherwise, it will use `tf.where` which will result in some redundant work
-  (both branches will be computed but only one selected). However, the tensors
-  involved will usually be small (means and variances in batchnorm), so the
-  cost will be small and will not be incurred at all if `pred` is a constant.
-
-  Args:
-    pred: A boolean scalar `Tensor`.
-    fn_then: A callable to use when pred==True.
-    fn_else: A callable to use when pred==False.
-
-  Returns:
-    A `Tensor` whose value is fn_then() or fn_else() based on the value of pred.
-  """
-  pred_value = utils.constant_value(pred)
-  if pred_value:
-    return fn_then()
-  elif pred_value is False:
-    return fn_else()
-  t_then = array_ops.expand_dims(fn_then(), 0)
-  t_else = array_ops.expand_dims(fn_else(), 0)
-  pred = array_ops.reshape(pred, [1])
-  result = array_ops.where(pred, t_then, t_else)
-  return array_ops.squeeze(result, [0])
